package Test::Package;

1;

__END__

=head1 NAME

Test::Package

=cut

=head1 DESCRIPTION

Some more text which can be formatted with tabs...

=cut

=head1 SYNOPSIS

Perhaps a code snippet

	use Test::Package;
	my $tp = Test::Package->new();

Add some more text because that is a thing

=cut

=head1 METHODS

=cut

=head2 Some Function

okay add some text

	SELECT thing
	FROM TABLE

=cut

=head2 other

add a list within

=over

=item one

=item two

=item three

=back

=cut

=head1 AUTHOR

LNATION C<< <email at lnation.org> >>

=cut

=head1 BUGS

Please report any bugs or feature requests to C<bug-test-package at rt.cpan.org>, or through
the web interface at L<https://rt.cpan.org/NoAuth/ReportBug.html?Queue=Test-Package>. I will
be notified, and then you'll automatically be notified of progress on your bug as I make changes.

=cut

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

	perldoc Test::Package

You can also look for information at:

=over

=item * RT: CPAN's request tracker (report bugs here)

L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-Package>

=item * Search CPAN

L<https://metacpan.org/release/Test-Package>

=back

=cut

=head1 ACKNOWLEDGEMENTS

=cut

=head1 LICENSE AND COPYRIGHT

This software is Copyright (c) 2022 by the author

This is free software, licensed under:

	The Artistic License 2.0 (GPL Compatible)

=cut